import { lamports, createGenericFileFromJson } from '@metaplex-foundation/umi';
import fetch from 'node-fetch';

/* eslint-disable no-await-in-loop */
const FormData = require('form-data');
const CASCADE_API_URL = 'https://gateway-api.pastel.network/';
function createCascadeUploader(context, options = {
  apiKey: ''
}) {
  const {
    apiKey
  } = options;
  if (!apiKey) {
    throw new Error('Cascade Gateway API key is required');
  }
  const getUploadPrice = async () => lamports(0);
  const upload = async files => {
    const uris = [];
    const body = new FormData();
    files.forEach(file => {
      body.append('files', Buffer.from(file.buffer), file.fileName);
    });
    try {
      const res = await fetch(`${CASCADE_API_URL}/api/v1/cascade?make_publicly_accessible=true`, {
        headers: {
          Api_key: apiKey
        },
        method: 'POST',
        body
      });
      const data = await res.json();
      data.results.forEach(item => {
        if (item.original_file_ipfs_link) uris.push(item.original_file_ipfs_link);else {
          uris.push('');
        }
      });
    } catch (e) {
      return [];
    }
    // eslint-disable-next-line no-console
    console.log(uris);
    return uris;
  };
  const upload2 = async files => {
    const body = new FormData();
    files.forEach(file => {
      body.append('files', Buffer.from(file.buffer), file.fileName);
    });
    try {
      const res = await fetch(`${CASCADE_API_URL}/api/v1/cascade?make_publicly_accessible=true`, {
        headers: {
          Api_key: apiKey
        },
        method: 'POST',
        body
      });
      const data = await res.json();
      return data.results;
    } catch (e) {
      return [];
    }
  };
  const uploadJson = async json => {
    const file = createGenericFileFromJson(json);
    const uris = await upload([file]);
    return uris[0];
  };
  return {
    getUploadPrice,
    upload,
    uploadJson,
    upload2
  };
}

export { createCascadeUploader };
//# sourceMappingURL=createCascadeUploader.mjs.map
